function [vt, om, vn] = TriskarOdometry(R_robot, R_wheel, alpha, v_F, v_L, omega, plot)

% - R_robot: robot radius (by mean of distance of the wheels from the center of the robot) [mm]
% 25 cm in our case -- 250 mm
% - R_wheel: wheel radius [mm]
%  24.6000 mm yaricap
% - alpha: angle of the front wheels [rads] (in our case alpha = pi/6)
% - vF: frontal speed setpoint [mm/s]    ---x axis 
% - vL: lateral speed setpoint [mm/s], > 0 if oriented to right ---y axis 
% - omega: angular velocity setpoint [rad/s] > 0 if CCW(counter clock wise)
% - plot: draw a plot with all vectors (1 = plot, 0 = don't plot)
% 
% this function returns:
% - vt: vector of tangential wheel velocities [mm/s]
% - vn: vector of "sliding" velocity of the wheels due to rollers [mm/s], > 0 if directed from the center to the extern
% - omega_w: angular velocity of the wheels [mm/s], > 0 if CW looking at the wheel from the center of the robot
% 
% You can call that function like:
% [vt,omega_w,vn] = TriskarOdometry(250, 24.6, pi/6,1000,1000,0,1)

cosA = cos(alpha);
sinA = sin(alpha);

v = [v_F, v_L, omega*R_robot]';

MF = [-cosA sinA -1;
       cosA sinA -1;
       0	-1	 -1];

ML = [sinA  cosA 0;
      sinA -cosA 0;
      -1	0	 0];

vt = MF*v;
om = vt/R_wheel;
vn = ML*v;

if(plot == 1)
    % scalature
    k1 = max(abs(v_F),abs(v_L));
    v1 = v_F/k1*R_robot/2;
    
    v2 = v_L/k1*R_robot/2;
    
    p1 = R_robot/2;
    
    m_arr = [0 0.04*p1 -0.04*p1; 0.1*p1 0 0];
    v_txt   = [0.05*p1; 0.2*p1];
    
    
    
    s1 = R_robot/( 2 * max(abs([vt;vn])));
    
    %plot
     figure(1)

    hold on
    h1 = line([0,R_robot*cosA],[0 R_robot*sinA]);
    h2 = line([0,-R_robot*cosA],[0 R_robot*sinA]);
    h3 = line([0,0],[0 -R_robot]);
    set(h1,'Color',[0.2 0.2 0.2]);
    set(h2,'Color',[0.2 0.2 0.2]);
    set(h3,'Color',[0.2 0.2 0.2]);
    set(h1,'LineStyle','-.');
    set(h2,'LineStyle','-.');
    set(h3,'LineStyle','-.');
    
    if(v_F ~= 0)
        line([0 0],[0 v1],'Color',[1 0 0]);
        if(v_F < 0)
            fill([0 p1*0.05 -p1*0.05],[v1 v1+0.12*p1 v1+0.12*p1],'r')
        else
            fill([0 p1*0.05 -p1*0.05],[v1 v1-0.12*p1 v1-0.12*p1],'r')
        end
        text(0.15*v1,v1,'v_F');
    end
    if(v_L ~= 0)
        line([0 v2],[0 0],'Color',[1 0 0]);
        if(v_L < 0)
            fill([v2 v2+0.12*p1 v2+0.12*p1],[0 p1*0.05 -p1*0.05],'r')
        else
            fill([v2 v2-0.12*p1 v2-0.12*p1],[0 p1*0.05 -p1*0.05],'r')
        end
        
        text(v2,0.1*v2,'v_L');
    end
    if(omega ~= 0)
        if(omega > 0)
            theta = linspace(-pi/3,pi/2,100);
            fill([-0.12*p1 0 0],[R_robot/6 R_robot/6+p1*0.05 R_robot/6-p1*0.05],'b');
            text(-0.14*p1,R_robot/6+p1*0.05,'\omega')
        else
            theta = linspace(pi/2,4*pi/3,100);
            fill([0.12*p1 0 0],[R_robot/6 R_robot/6+p1*0.05 R_robot/6-p1*0.05],'b');
            text(0.12*p1,R_robot/6+p1*0.05,'\omega');
        end
        rho = ones(1,100)*R_robot/6;
        [xr,yr] = pol2cart(theta,rho);
        line(xr,yr);
    end
    % ruota 1
    
    if(vt(1) ~= 0)
    line([R_robot*cosA, R_robot*cosA+vt(1)*s1*sinA],[R_robot*sinA, R_robot*sinA-vt(1)*s1*cosA],'Color',[0 1 0]);
    offset1 = [R_robot*cosA+vt(1)*s1*sinA; R_robot*sinA-vt(1)*s1*cosA];
    if(vt(1)*s1*sinA < 0)
        M = m_rot(alpha);
    else
        M = m_rot(-pi + alpha);
    end
    
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{t1}');
    end
    
    if(vn(1) ~= 0)
    line([R_robot*cosA, R_robot*cosA+vn(1)*s1*cosA],[R_robot*sinA, R_robot*sinA+vn(1)*s1*sinA],'Color',[0 1 0])
    offset1 = [R_robot*cosA+vn(1)*s1*cosA; R_robot*sinA+vn(1)*s1*sinA];
    if(vn(1)*s1*cosA < 0)
        M = m_rot(pi/2+alpha);
    else
        M = m_rot(-pi/2 + alpha);
    end
    
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{n1}');
    end
    
    % ruota 2
    if(vt(2) ~= 0)
    line([-R_robot*cosA, -R_robot*cosA+vt(2)*s1*sinA],[R_robot*sinA, R_robot*sinA+vt(2)*s1*cosA],'Color',[0 1 0]);
    offset1 = [-R_robot*cosA+vt(2)*s1*sinA; R_robot*sinA+vt(2)*s1*cosA];
    if(vt(2)*s1*sinA < 0)
        M = m_rot(-pi-alpha);
    else
        M = m_rot(-alpha);
    end
    
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{t2}');
    end
    
    if(vn(2) ~= 0)
    line([-R_robot*cosA, -R_robot*cosA-vn(2)*s1*cosA],[R_robot*sinA, R_robot*sinA+vn(2)*s1*sinA],'Color',[0 1 0])
    offset1 = [-R_robot*cosA-vn(2)*s1*cosA; R_robot*sinA+vn(2)*s1*sinA];
    if(vn(2)*s1*sinA < 0)
        M = m_rot(-pi/2-alpha);
    else
        M = m_rot(pi/2-alpha);
    end
    
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{n2}');
    end
    
    % ruota 3
    if(vt(3) ~= 0)
    line([0, -vt(3)*s1],[-R_robot, -R_robot],'Color',[0 1 0]);
    offset1 = [-vt(3)*s1; -R_robot];
    if(-vt(3)*s1 < 0)
        M = m_rot(pi/2);
    else
        M = m_rot(-pi/2);
    end
    
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{t3}');
    end
    
    if(vn(3) ~= 0)
    line([0, 0],[-R_robot, -R_robot-vn(3)*s1],'Color',[0 1 0])
    offset1 = [0; -R_robot-vn(3)*s1];
    if(-vn(3)*s1 < 0)
        M = m_rot(pi);
    else
        M = m_rot(0);
    end
    m_arr1 = M*m_arr + [offset1 offset1 offset1];
    v_txt1 = M*v_txt + offset1;
    fill(m_arr1(1,:),m_arr1(2,:),'g');
    text(v_txt1(1),v_txt1(2),'v_{n3}');
    end
    
    y_ref = 0;
    delta = -0.095*R_robot;
    text(R_robot*0.6,y_ref,['v_{t1}: ',num2str(vt(1)),' mm/s'])
    text(-R_robot*0.8,y_ref,['v_{F}: ',num2str(v_F),' mm/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['v_{n1}: ',num2str(vn(1)),' mm/s'])
    text(-R_robot*0.8,y_ref,['v_{L}: ',num2str(v_L),' mm/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['v_{t2}: ',num2str(vt(2)),' mm/s'])
    text(-R_robot*0.8,y_ref,['\omega_{R}: ',num2str(omega),' rad/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['v_{n2}: ',num2str(vn(2)),' mm/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['v_{t3}: ',num2str(vt(3)),' mm/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['v_{n3}: ',num2str(vn(3)),' mm/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['\omega_1: ',num2str(om(1)),' rad/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['\omega_2: ',num2str(om(2)),' rad/s'])
    y_ref = y_ref + delta;
    text(R_robot*0.6,y_ref,['\omega_3: ',num2str(om(3)),' rad/s'])
    
    axis equal
    hold off
end






   